﻿using ClassAttendanceStatistics.Model;
using GalaSoft.MvvmLight.Command;
using GS.Terminal.Common.WebService;
using GS.Terminal.DashBoard.Interface;
using GS.Unitive.Framework.Core;
using GS.Unitive.Framework.Logging;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Develop.Extension.Wpf;

namespace ClassAttendanceStatistics
{
    public class CourseAttendanceViewModel : PageViewModel
    {
        #region 字段
        private static ILogWriter logger = AddonRuntime.Instance.LogWriter;
        #endregion

        #region 方法
        public CourseAttendanceViewModel()
        {
            DayPager.OnPageChanged += DayPager_OnPageChanged;
        }

        public override void Enter()
        {

        }

        public override void Leave()
        {

        }

        public override bool Update()
        {
            return LoadData();
        }

        private bool LoadData()
        {
            var postResp = WebServiceCenter.Post<WebServiceResult<List<ResponseModel.ResponseModel>>>(new Uri(Source), new object());

            //数据出错
            if (postResp.HasError())
            {
                logger.Error(postResp.String4Log, null);
                return false;
            }
            //数据正常
            else
            {
                ConvertModel(postResp.result);
                return true;
            }
        }

        /// <summary>
        /// 将数据模型转换为视图模型
        /// </summary>
        /// <param name="result"></param>
        private void ConvertModel(List<ResponseModel.ResponseModel> result)
        {
            LessonList = new ObservableCollection<Model.LessonInfo>(result.OrderBy(s => s.CourseName).Select(s => (LessonInfo)s));
            HasContent = (LessonList.Count == 0 ? false : true);


            DayPager.TotalData = LessonList.Count;


            if (HasContent && SelectedLesson == null)
            {
                SelectLessonCommand?.Execute(LessonList.FirstOrDefault());
            }

        }

        private void DayPager_OnPageChanged(Pager obj)
        {
            ObservableCollection<LessonInfo> lessons = new ObservableCollection<LessonInfo>();

            LessonList.OrderBy(s => s.CourseName).Skip(DayPager.PageSize * (DayPager.PageIndex - 1)).Take(7).ToList().ForEach((s) =>
            {
                lessons.Add(s);
            });

            //美工太狠毒了, 界面实在不会做了, 只能通过数据解决
            if (lessons.Count < 7)
            {
                int addCount = 7 - lessons.Count;

                for (int i = 0; i < addCount; i++)
                {
                    lessons.Add(new LessonInfo());
                }
            }

            LessonListShow = lessons;

            SelectLessonCommand.Execute(LessonListShow?.FirstOrDefault());
        }

        #endregion

        #region View绑定
        private Pager _DayPager;

        public Pager DayPager
        {
            get { return _DayPager ?? (_DayPager = new Pager(7)); }
            set { _DayPager = value; RaisePropertyChanged("DayPager"); }
        }

        private ObservableCollection<LessonInfo> _LessonListShow;
        /// <summary>
        /// 用于显示当前课程列表
        /// </summary>
        public ObservableCollection<LessonInfo> LessonListShow
        {
            get { return _LessonListShow ?? (_LessonListShow = new ObservableCollection<LessonInfo>()); }
            set { _LessonListShow = value; RaisePropertyChanged("LessonListShow"); }
        }

        private bool _HasContent;
        /// <summary>
        /// 是否有课程信息
        /// </summary>
        public bool HasContent
        {
            get { return _HasContent; }
            set { _HasContent = value; RaisePropertyChanged("HasContent"); }
        }

        private ObservableCollection<ObservableCollection<ClassItem>> _ClassListShow;
        /// <summary>
        /// 中间List显示的课程信息
        /// </summary>
        public ObservableCollection<ObservableCollection<ClassItem>> ClassListShow
        {
            get { return _ClassListShow ?? (_ClassListShow = new ObservableCollection<ObservableCollection<ClassItem>>()); }
            set { _ClassListShow = value; RaisePropertyChanged("ClassListShow"); }
        }
        #endregion

        #region 数据支持
        private LessonInfo _SelectedLesson;
        /// <summary>
        /// 当前课程中的所有信息
        /// </summary>
        public LessonInfo SelectedLesson
        {
            get { return _SelectedLesson; }
            set { _SelectedLesson = value; RaisePropertyChanged("SelectedLesson"); }
        }

        private ObservableCollection<LessonInfo> _LessonList;
        /// <summary>
        /// 所有课程
        /// </summary>
        public ObservableCollection<LessonInfo> LessonList
        {
            get { return _LessonList ?? (_LessonList = new ObservableCollection<LessonInfo>()); }
            set { _LessonList = value; RaisePropertyChanged("LessonList"); }
        }
        #endregion

        #region 命令
        public RelayCommand<LessonInfo> SelectLessonCommand => new RelayCommand<LessonInfo>((lessoninfo) =>
        {
            if (lessoninfo != null && !lessoninfo.IsSelected && lessoninfo.ClassList != null)
            {
                if (SelectedLesson != null) SelectedLesson.IsSelected = false;
                lessoninfo.IsSelected = true;
                SelectedLesson = lessoninfo;

                //分页后显示到主界面
                if (SelectedLesson.ClassList != null)
                {
                    ClassListShow = PageControl.CutPage(SelectedLesson.ClassList, 14);
                }
            }
        });
        #endregion
    }
}
