﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight;
using System.Windows.Media;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Controls;
using GalaSoft.MvvmLight.Command;
using System.Windows.Media.Animation;
using GS.Terminal.DashBoard.Interface;
using System.Runtime.InteropServices;

namespace DashBoardViewModel
{
    public partial class DashBoardMain
    {
        private int minX = -(int)SystemParameters.PrimaryScreenWidth * 2;
        private int maxX = (int)SystemParameters.PrimaryScreenWidth * 2;
        private int WindowWidth = (int)SystemParameters.PrimaryScreenWidth;
        private int _TurnSecond = 10;

        public int TurnSecond
        {
            get { return _TurnSecond; }
            set
            {
                _TurnSecond = value;
                AutoTurnDownCount = value;
            }
        }

        /// <summary>
        /// 加载DashBoard到界面上
        /// </summary>
        /// <param name="list"></param>
        public void InitDashBoard(List<DashBoardItem> list)
        {
            AllDashBoard = list;
            ShowLRButton = AllDashBoard.Count > 1;
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                ViewList = new ObservableCollection<DashBoardItem>(list);
                if (ViewList.Count < 6)
                {
                    var count = 6 - ViewList.Count;
                    for (int i = 0; i < count; i++)
                    {
                        ViewList.Add(new DashBoardItem()
                        {
                            ID = Guid.Empty,
                            Title = "待开放",
                            ViewContent = null,
                            Icon = new Uri("images/icon-waiting.png", UriKind.Relative)
                        });
                    }
                }
            }));
            ViewConetnt.Init(list);
        }
        private List<DashBoardItem> AllDashBoard = new List<DashBoardItem>();
        private ObservableCollection<DashBoardItem> _ViewList;

        public ObservableCollection<DashBoardItem> ViewList
        {
            get { return _ViewList; }
            set { _ViewList = value; RaisePropertyChanged("ViewList"); }
        }

        private bool _AutoTurnEnable = true;

        public bool AutoTurnEnable
        {
            get { return _AutoTurnEnable; }
            set
            {
                _AutoTurnEnable = value;

                RaisePropertyChanged("AutoTurnEnable");
            }
        }

        private int _AutoTurnDownCount = 10;

        public int AutoTurnDownCount
        {
            get { return _AutoTurnDownCount; }
            set { _AutoTurnDownCount = value; RaisePropertyChanged("AutoTurnDownCount"); }
        }


        public void startTurn()
        {
            if (fullScreen && AutoTurnEnable && AllDashBoard.Count > 1)
            {
                ShowDownCountLabel = true;
                Task.Run(async () =>
                {
                    while (fullScreen && AutoTurnEnable)
                    {
                        await Task.Delay(1000);
                        if (GetIdleTick() >= 5000)
                            AutoTurnDownCount--;
                        else
                            AutoTurnDownCount = TurnSecond;
                        if (AutoTurnDownCount == 0)
                        {
                            AutoTurnDownCount = TurnSecond;
                            await Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                            {
                                moveView("left");
                            }));
                        }
                    }
                    AutoTurnDownCount = TurnSecond;
                });
            }
            else
            {
                ShowDownCountLabel = false;
            }
                
        }

        private bool _SwitchEnable = true;

        public bool SwitchEnable
        {
            get { return _SwitchEnable; }
            set { _SwitchEnable = value; RaisePropertyChanged("SwitchEnable"); }
        }

        private bool _fullScreen;

        public bool fullScreen
        {
            get { return _fullScreen; }
            set
            {
                _fullScreen = value;
                if (value)
                    startTurn();
                RaisePropertyChanged("fullScreen");
                RaisePropertyChanged("NoFullScreen");
            }
        }

        public bool NoFullScreen
        {
            get { return !_fullScreen; }
        }

        public RelayCommand<DashBoardItem> MenuItemClick => new RelayCommand<DashBoardItem>(item =>
        {
            ShowItem(item);
        });

        public void ShowItem(DashBoardItem dashBoardItem = null)
        {
            if (dashBoardItem == null)
                dashBoardItem = AllDashBoard.FirstOrDefault();
            if (dashBoardItem.ID == Guid.Empty) return;
            var selectIndex = AllDashBoard.IndexOf(dashBoardItem);//看一下当前点击的是第几个
            SecondFrame.ViewContent = null;
            SecondFrame.ViewContent = ViewConetnt.AllocContent(dashBoardItem.ID); //item.ViewContent;//.Clone() as IDashBoardPage;
            var Pindex = selectIndex - 1 < 0 ? AllDashBoard.Count - 1 : selectIndex - 1;//前面的
            var Nindex = selectIndex + 1 >= AllDashBoard.Count ? 0 : selectIndex + 1;
            FirstFrame.ViewContent = ViewConetnt.AllocContent(AllDashBoard[Pindex].ViewContent.PageID);//.Clone() as IDashBoardPage;
            ThirdFrame.ViewContent = ViewConetnt.AllocContent(AllDashBoard[Nindex].ViewContent.PageID);//.Clone() as IDashBoardPage;
            fullScreen = true;
        }

        /// <summary>
        /// 页面切换结束后  根据当前X为0的Frame重新计算Content
        /// </summary>
        private void MathFrameContent()
        {
            var NewFirstFrame = GetFrameByX(-WindowWidth);
            var NewSecondFrame = GetFrameByX(0);
            var NewThirdFrame = GetFrameByX(WindowWidth);
            var SecondFramePage = AllDashBoard.FirstOrDefault(ss => ss.ID == NewSecondFrame.Content.PageID);
            var selectIndex = AllDashBoard.IndexOf(SecondFramePage);
            var Pindex = selectIndex - 1 < 0 ? AllDashBoard.Count - 1 : selectIndex - 1;//前面的
            var Nindex = selectIndex + 1 >= AllDashBoard.Count ? 0 : selectIndex + 1;
            //(NewFirstFrame.Content)?.Dispose();
            //NewFirstFrame.ViewContent = null;
            NewFirstFrame.ViewContent = ViewConetnt.AllocContent(AllDashBoard[Pindex].ViewContent.PageID);//.Clone() as IDashBoardPage;
            //(NewThirdFrame.Content)?.Dispose();
            //NewThirdFrame.ViewContent = null;
            NewThirdFrame.ViewContent = ViewConetnt.AllocContent(AllDashBoard[Nindex].ViewContent.PageID);//.Clone() as IDashBoardPage;
        }

        private ViewFrame GetFrameByX(int X)
        {
            if (FirstFrame.X == X) return FirstFrame;
            else if (SecondFrame.X == X) return SecondFrame;
            else if (ThirdFrame.X == X) return ThirdFrame;
            else return null;
        }

        public RelayCommand CloseFullScreen => new RelayCommand(() =>
        {
            fullScreen = false;
            FirstFrame.Content = null;
            SecondFrame.Content = null;
            ThirdFrame.Content = null;
            ResetFrameLocation();
        });
        /// <summary>
        /// 重置Frame位置
        /// </summary>
        private void ResetFrameLocation()
        {
            FirstFrame.X = -WindowWidth;
            FirstFrame.Reset();
            SecondFrame.X = 0;
            SecondFrame.Reset();
            ThirdFrame.X = WindowWidth;
            ThirdFrame.Reset();
        }

        #region 用于视图切换的三个Frame
        private ViewFrame _FirstFrame;

        public ViewFrame FirstFrame
        {
            get { return _FirstFrame ?? (_FirstFrame = new ViewFrame(-WindowWidth)); }
            set { _FirstFrame = value; RaisePropertyChanged("FirstFrame"); }
        }

        private ViewFrame _SecondFrame;

        public ViewFrame SecondFrame
        {
            get { return _SecondFrame ?? (_SecondFrame = new ViewFrame(0)); }
            set { _SecondFrame = value; RaisePropertyChanged("SecondFrame"); }
        }

        private ViewFrame _ThirdFrame;

        public ViewFrame ThirdFrame
        {
            get { return _ThirdFrame ?? (_ThirdFrame = new ViewFrame(WindowWidth)); }
            set { _ThirdFrame = value; RaisePropertyChanged("ThirdFrame"); }
        }
        #endregion
        private QuinticEase AnimationFunction = new QuinticEase();
        private Duration animationDuration = new Duration(TimeSpan.FromMilliseconds(1000));

        public RelayCommand<string> Move => new RelayCommand<string>(direction =>
        {
            moveView(direction);
            AutoTurnDownCount = TurnSecond;
        });

        private void moveView(string direction)
        {
            SwitchEnable = false;
            if (direction == "left")
            {
                FirstFrame.X -= WindowWidth;
                SecondFrame.X -= WindowWidth;
                ThirdFrame.X -= WindowWidth;
            }
            else if (direction == "right")
            {
                FirstFrame.X += WindowWidth;
                SecondFrame.X += WindowWidth;
                ThirdFrame.X += WindowWidth;
            }
            else return;
            DoubleAnimation firstAnimation = new DoubleAnimation(0, animationDuration);
            firstAnimation.To = FirstFrame.X;
            firstAnimation.Completed += FirstAnimation_Completed;
            firstAnimation.EasingFunction = AnimationFunction;
            DoubleAnimation secondAnimation = new DoubleAnimation(0, animationDuration);
            secondAnimation.To = SecondFrame.X;
            secondAnimation.Completed += SecondAnimation_Completed;
            secondAnimation.EasingFunction = AnimationFunction;
            DoubleAnimation thirdAnimation = new DoubleAnimation(0, animationDuration);
            thirdAnimation.To = ThirdFrame.X;
            thirdAnimation.Completed += ThirdAnimation_Completed;
            thirdAnimation.EasingFunction = AnimationFunction;

            FirstFrame.Transform.BeginAnimation(TranslateTransform.XProperty, firstAnimation);
            beginAnimation();
            SecondFrame.Transform.BeginAnimation(TranslateTransform.XProperty, secondAnimation);
            beginAnimation();
            ThirdFrame.Transform.BeginAnimation(TranslateTransform.XProperty, thirdAnimation);
            beginAnimation();
        }

        private void ThirdAnimation_Completed(object sender, EventArgs e)
        {
            SwitchEnable = true;
            bool reset = false;
            if (ThirdFrame.X == minX)
            {
                ThirdFrame.X = WindowWidth;
                reset = true;
            }
            else if (ThirdFrame.X == maxX)
            {
                ThirdFrame.X = -WindowWidth;
                reset = true;
            }
            if (reset)
                ThirdFrame.Reset();
            endAnimation();
        }

        private void SecondAnimation_Completed(object sender, EventArgs e)
        {
            SwitchEnable = true;
            bool reset = false;
            if (SecondFrame.X == minX)
            {
                SecondFrame.X = WindowWidth;
                reset = true;
            }
            else if (SecondFrame.X == maxX)
            {
                SecondFrame.X = -WindowWidth;
                reset = true;
            }
            if (reset)
                SecondFrame.Reset();
            endAnimation();
        }

        private void FirstAnimation_Completed(object sender, EventArgs e)
        {
            SwitchEnable = true;
            bool reset = false;
            if (FirstFrame.X == minX)
            {
                FirstFrame.X = WindowWidth;
                reset = true;
            }
            else if (FirstFrame.X == maxX)
            {
                FirstFrame.X = -WindowWidth;
                reset = true;
            }
            if (reset)
                FirstFrame.Reset();

            endAnimation();
        }
        private List<Guid> syncList = new List<Guid>();
        private void beginAnimation()
        {
            syncList.Add(Guid.NewGuid());
        }

        private void endAnimation()
        {
            syncList.RemoveAt(0);
            if (syncList.Count == 0)
            {
                MathFrameContent();
            }
        }

        public void SetDashBoardThumbnail(Uri uri, Guid viewID)
        {
            var view =
            ViewList.FirstOrDefault(ss => ss.ID == viewID);
            if (view != null)
            {
                view.Thumbnail = new BitmapImage(uri);
            }

        }


        /// <summary>
        /// 获取最后操作时间与当前时间的间隔（毫秒）
        /// </summary>
        /// <returns></returns>
        public static long GetIdleTick()
        {
            LASTINPUTINFO lastInputInfo = new LASTINPUTINFO();
            lastInputInfo.cbSize = Marshal.SizeOf(lastInputInfo);
            if (!GetLastInputInfo(ref lastInputInfo)) return 0;
            return Environment.TickCount - (long)lastInputInfo.dwTime;
        }
        [DllImport("user32.dll")]
        static extern bool GetLastInputInfo(ref LASTINPUTINFO plii);

        [StructLayout(LayoutKind.Sequential)]
        private struct LASTINPUTINFO
        {
            [MarshalAs(UnmanagedType.U4)]
            public int cbSize;
            [MarshalAs(UnmanagedType.U4)]
            public uint dwTime;
        }
    }

    public class DashBoardItem : ObservableObject
    {

        public Guid ID { get; set; }

        private ImageSource _Thumbnail;
        /// <summary>
        /// 缩略图
        /// </summary>
        public ImageSource Thumbnail
        {
            get
            {
                if (_Thumbnail == null)
                {
                    _Thumbnail = new BitmapImage(new Uri("/DashBoardView;component/images/img-waiting.jpg", UriKind.Relative));
                }
                return _Thumbnail;
            }
            set { _Thumbnail = value; RaisePropertyChanged("Thumbnail"); }
        }

        private Uri _Icon;

        public Uri Icon
        {
            get { return _Icon; }
            set { _Icon = value; RaisePropertyChanged("Icon"); }
        }

        private string _Title;

        public string Title
        {
            get { return _Title; }
            set { _Title = value; RaisePropertyChanged("Title"); }
        }

        private IDashBoardPage _ViewContent;

        public IDashBoardPage ViewContent
        {
            get { return _ViewContent; }
            set
            {
                _ViewContent = value;
                RaisePropertyChanged("ViewContent");
                RaisePropertyChanged("HasContent");
                RaisePropertyChanged("NoContent");
            }
        }

        public bool HasContent
        {
            get { return ViewContent != null; }
        }

        public bool NoContent
        {
            get { return ViewContent == null; }
        }
    }

    public class ViewFrame : ObservableObject
    {
        public ViewFrame(int initx)
        {
            X = initx;
            Reset();
        }

        private int _x = int.MaxValue;

        public int X
        {
            get { return _x; }
            set
            {
                _x = value;
            }
        }

        private IDashBoardPage _Content;

        public IDashBoardPage Content
        {
            get { return _Content; }
            set
            {
                _Content = value;
                RaisePropertyChanged("Content");
            }
        }

        private ViewConetnt _ViewContent;

        public ViewConetnt ViewContent
        {
            get { return _ViewContent; }
            set
            {
                ViewConetnt.ReleaseContent(_ViewContent);
                if (value == null)//释放
                {
                    Content = null;
                }
                else
                {

                    Content = value.Content;
                }
                _ViewContent = value;
                RaisePropertyChanged("ViewContent");
            }
        }


        private Transform _Transform;

        public Transform Transform
        {
            get { return _Transform; }
            set { _Transform = value; RaisePropertyChanged("Transform"); }
        }

        public void Reset()
        {
            this.Transform = new TranslateTransform(X, 0);
        }
    }

    public class ViewConetnt
    {
        private static List<ViewConetnt> ContentList = new List<ViewConetnt>();
        public static void Init(List<DashBoardItem> list)
        {
            foreach (var item in list)
            {
                ContentList.Add(new ViewConetnt()
                {
                    ContentID = item.ID,
                    Content = item.ViewContent.Clone() as IDashBoardPage
                });
            }

        }
        public static ViewConetnt AllocContent(Guid contentID)
        {
            var content = ContentList.FirstOrDefault(ss => ss.ContentID == contentID && !ss.IsLock);
            if (content == null)//如果没有空闲资源，需要重新申请一块
            {
                var c = ContentList.FirstOrDefault(ss => ss.ContentID == contentID);
                if (c != null)
                {
                    var nc = new ViewConetnt()
                    {
                        ContentID = c.ContentID,
                        Content = c.Content.Clone() as IDashBoardPage
                    };
                    ContentList.Add(nc);
                    content = nc;
                }
            }
            if (content != null)
                content.IsLock = true;
            return content;
        }

        public static void ReleaseContent(ViewConetnt vc)
        {
            if (vc != null)
            {
                vc.IsLock = false;
            }
        }

        public ViewConetnt()
        {
            ID = Guid.NewGuid();
        }
        public IDashBoardPage Content { get; set; }

        public Guid ContentID { get; set; }

        public Guid ID { get; private set; }

        public bool IsLock { get; set; }
    }
}
