﻿using DashBoardViewModel;
using GS.Terminal.DashBoard.Interface;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GS.Terminal.DashBoard.Logic.Core
{
    public class DashBoardCore
    {
        public DashBoardCore(IDashBoardPage[] dashBoardPages)
        {
            if (dashBoardPages != null)
                foreach (var item in dashBoardPages)
                {
                    var db = new Models.DashBoard();
                    db.ID = item.PageID;
                    db.Page = item;
                    db.ViewModel = item.BindDataContext();
                    var config = Program.setting.DashBoardConfig.Datas?.FirstOrDefault(ss => ss.DashBoardID == item.PageID);
                    if (config != null)
                    {
                        db.OrderIndex = config.OrderIndex;
                        db.Enable = config.Enable;
                        db.ViewModel.Source = config.Source;
                    }
                    DashBoards.Add(db);
                }
        }

        public List<Models.DashBoard> DashBoards { get; set; } = new List<Models.DashBoard>();

        /// <summary>
        /// 将试图加载到界面
        /// </summary>
        public void doView()
        {
            var viewItems = DashBoards.Where(ss => ss.Enable)
                .OrderBy(ss => ss.OrderIndex)
                .Select(ss => new DashBoardItem()
                {
                    ID = ss.Page.PageID,
                    Icon = ss.Page.Icon,
                    Title = ss.Page.Title,
                    ViewContent = ss.Page
                }).ToList();
            Program.locator.Main.InitDashBoard(viewItems);
        }

        private Task refreshTask;
        /// <summary>
        /// 开始刷新数据
        /// </summary>
        public void StartRefresh()
        {
            if (refreshTask == null)
            {
                refreshTask = new Task(async () =>
                {
                    IsRefreshing = true;
                    do
                    {
                        DashBoards
                        .Where(ss => ss.Enable)
                        .ToList()
                        .ForEach(ss =>
                                  {
                                      try
                                      {
                                          ss.ViewModel.Update();
                                      }
                                      catch (Exception ee)
                                      {
                                          Program.logger.Error($"DashBoard数据刷新异常。", ee);
                                      }
                                  });
                        await Task.Delay(1000 * 60);
                    } while (IsRefreshing);
                });
            }
            if (!IsRefreshing)
                refreshTask.Start();
        }

        /// <summary>
        /// 根据视图ID获取试图的ViewModel
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public PageViewModel GetViewModelByID(Guid viewid)
        {
            return DashBoards.FirstOrDefault(ss => ss.ID == viewid)?.ViewModel;
        }

        #region Property

        private bool _IsRefreshing;
        /// <summary>
        /// 数据刷新线程是否正在运行中
        /// </summary>
        public bool IsRefreshing
        {
            get { return _IsRefreshing; }
            set { _IsRefreshing = value; }
        }


        private int _SwitchSpan;
        /// <summary>
        /// 切换间隔
        /// </summary>
        public int SwitchSpan
        {
            get { return _SwitchSpan; }
            set { _SwitchSpan = value; }
        }


        private bool _AutoSwitch;
        /// <summary>
        /// 是否自动播放
        /// </summary>
        public bool AutoSwitch
        {
            get { return _AutoSwitch; }
            set
            {
                _AutoSwitch = value;
            }
        }


        private int _AutoCloseSecond;
        /// <summary>
        /// 自动关闭时间
        /// </summary>
        public int AutoCloseSecond
        {
            get { return _AutoCloseSecond; }
            set { _AutoCloseSecond = value; }
        }
        #endregion

    }
}
