﻿using GalaSoft.MvvmLight.Threading;
using GS.Unitive.Framework.Core;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace GS.Terminal.DashBoard
{
    /// <summary>
    /// App.xaml 的交互逻辑
    /// </summary>
    public partial class App : Application
    {
        public App()
        {
            DispatcherHelper.Initialize();
        }
        private void Application_Startup(object sender, StartupEventArgs e)
        {
            Environment.CurrentDirectory = System.Windows.Forms.Application.StartupPath;
            #region 检查重复启动
            Process[] pro = Process.GetProcesses();
            var process = pro.Where(p => p.ProcessName.Equals(Process.GetCurrentProcess().ProcessName));
            int n = process.Count();
            if (n > 1)
            {
                MessageBox.Show("程序已经启动，请勿重复启动", "提示", MessageBoxButton.OK, MessageBoxImage.Error);
                HandleRunningInstance(process.FirstOrDefault());
                Current.Shutdown();
                return;
            }
            #endregion
            try
            {
                #region 启动更新器
                //更新AutoUpdate
                string newUpdaterPath = System.IO.Path.Combine(System.Windows.Forms.Application.StartupPath, "AutoUpdate.exe.replace");
                string oldUpdatePath = System.IO.Path.Combine(System.Windows.Forms.Application.StartupPath, "AutoUpdate.exe");
                if (System.IO.File.Exists(newUpdaterPath))
                {
                    var updateProcess = pro.FirstOrDefault(ss => ss.ProcessName == "AutoUpdate");
                    if (updateProcess != null) updateProcess.Kill();
                    System.IO.File.Copy(newUpdaterPath, oldUpdatePath, true);
                    System.IO.File.Delete(newUpdaterPath);
                }



                string updateUrl = ConfigurationManager.AppSettings.Get("Updateurl");
                if (!string.IsNullOrEmpty(updateUrl))
                {
                    Process updateProcess = Process.Start("AutoUpdate.exe", String.Format("{0} {1}", updateUrl, this.GetType().Assembly.GetName().Name));
                    updateProcess.WaitForExit();
                }
                #endregion
            }
            catch { }

            #region 启动插件
            AddonRuntime addonRuntime = new AddonRuntime();
            try
            {
                HandleRunningInstance(process.FirstOrDefault());
                DispatcherUnhandledException += new System.Windows.Threading.DispatcherUnhandledExceptionEventHandler(App_DispatcherUnhandledException);
                AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
                LaunchPage launcherPage = new LaunchPage();
                Thread thread = new Thread(new ThreadStart(() =>
                {
                    //launcherPage = new LaunchPage();
                    addonRuntime.onAddonActive += new Unitive.Framework.Event.AddonStateChangeHandler(launcherPage.runtime_onAddonActive);
                    addonRuntime.onAddonInstall += new Unitive.Framework.Event.AddonStateChangeHandler(launcherPage.runtime_onAddonInstall);
                    addonRuntime.onAddonStarting += new Unitive.Framework.Event.AddonStateChangeHandler(launcherPage.runtime_onAddonStarting);
                    addonRuntime.onRuntimeCompleted += new Unitive.Framework.Event.RuntimeEventHandler(launcherPage.addonRuntime_onRuntimeCompleted);
                    Current.Dispatcher.BeginInvoke(new Action(launcherPage.Show));
                    System.Windows.Threading.Dispatcher.Run();
                }))
                { IsBackground = true };
                thread.SetApartmentState(ApartmentState.STA);
                thread.Start();
                addonRuntime.Start();
                Window mainWindow = AddonRuntime.Instance.GetFirstDefaultHostMain<Window>() ?? throw new Exception("未找到程序入口界面"); ;
                MainWindow.ContentRendered += (s, ee) =>
                {
                    //Current.Dispatcher.BeginInvoke(new Action(launcherPage.Close));
                    launcherPage.Close();
                };
                mainWindow?.Show();

            }
            catch (Exception ee)
            {
                AddonRuntime.Instance.LogWriter.Error("程序加载错误", ee);
                MessageBox.Show(ee.Message);
                Application.Current.Shutdown();
            }
            #endregion
        }


        void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            AddonRuntime.Instance.LogWriter.Error("CurrentDomain_UnhandledException发生未处理异常", (Exception)e.ExceptionObject);
        }

        void App_DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            AddonRuntime.Instance.LogWriter.Error("App_DispatcherUnhandledException发生未处理异常", e.Exception);
            e.Handled = true;
        }

        ///<summary>
        /// 该函数设置由不同线程产生的窗口的显示状态
        /// </summary>
        /// <param name="hWnd">窗口句柄</param>
        /// <param name="cmdShow">指定窗口如何显示。查看允许值列表，请查阅ShowWlndow函数的说明部分</param>
        /// <returns>如果函数原来可见，返回值为非零；如果函数原来被隐藏，返回值为零</returns>
        [DllImport("User32.dll")]
        private static extern bool ShowWindowAsync(IntPtr hWnd, int cmdShow);


        /// <summary>
        ///  该函数将创建指定窗口的线程设置到前台，并且激活该窗口。键盘输入转向该窗口，并为用户改各种可视的记号。
        ///  系统给创建前台窗口的线程分配的权限稍高于其他线程。 
        /// </summary>
        /// <param name="hWnd">将被激活并被调入前台的窗口句柄</param>
        /// <returns>如果窗口设入了前台，返回值为非零；如果窗口未被设入前台，返回值为零</returns>
        [DllImport("User32.dll")]
        private static extern bool SetForegroundWindow(IntPtr hWnd);



        private const int SW_SHOWNOMAL = 1;
        internal static void HandleRunningInstance(Process instance)
        {
            ShowWindowAsync(instance.MainWindowHandle, SW_SHOWNOMAL);//显示
            SetForegroundWindow(instance.MainWindowHandle);//当到最前端
        }
    }
}
