﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;

namespace SchoolSituation
{
    /// <summary>
    /// 房间对象
    /// </summary>
    public class ViewRoom : ObservableObject
    {
        /// <summary>
        /// 房间ID
        /// </summary>
        private int _RoomID;
        public int RoomID
        {
            get { return _RoomID; }
            set { _RoomID = value; RaisePropertyChanged("RoomID"); }
        }
        /// <summary>
        /// 房间名称
        /// </summary>
        private string _RoomName;
        public string RoomName
        {
            get { return _RoomName; }
            set { _RoomName = value; RaisePropertyChanged("RoomName"); }
        }
        /// <summary>
        /// 预约信息
        /// </summary>
        private List<TimeSlot> _orderlist;
        public List<TimeSlot> orderlist
        {
            get { return _orderlist; }
            set { _orderlist = value; RaisePropertyChanged("orderlist"); }
        }

        public static explicit operator ViewRoom(DashBoard.ServiceModel.Room data)
        {
            var entity = new ViewRoom();

            entity.RoomID = data.RoomID;
            entity.RoomName = data.RoomName;
            entity.orderlist = data.slot.Select(r => (TimeSlot)r).ToList(); ;
            return entity;
        }
    }
    public class TimeSlot : ObservableObject
    {
        /// <summary>
        /// 预约时间段ID
        /// </summary>
        private string _DayTimeId;
        public string DayTimeID
        {
            get { return _DayTimeId; }
            set { _DayTimeId = value; RaisePropertyChanged("DayTimeID"); }
        }
        /// <summary>
        /// 预约时间段
        /// </summary>
        private string _DayTime;

        public string DayTime
        {
            get { return _DayTime; }
            set { _DayTime = value; RaisePropertyChanged("DayTime"); }
        }
        /// <summary>
        /// 是否可预约，true可预约false不可预约
        /// </summary>
        private bool _IsOrder;

        public bool IsOrder
        {
            get { return _IsOrder; }
            set { _IsOrder = value; RaisePropertyChanged("IsOrder"); }
        }

        /// <summary>
        /// 预约信息
        /// </summary>
        private ViewOrder _order;
        public ViewOrder order
        {
            get { return _order; }
            set { _order = value; RaisePropertyChanged("order"); }
        }
        public static explicit operator TimeSlot(DashBoard.ServiceModel.TimeSlot data)
        {
            var entity = new TimeSlot();
            entity.DayTimeID = data.DayTimeID;
            entity.DayTime = data.DayTime;
            entity.IsOrder = !data.IsTrue;
            entity.order = (ViewOrder)data.order;

            return entity;
        }
    }
    /// <summary>
    /// 预约信息
    /// </summary>
    public class ViewOrder : ObservableObject
    {
        /// <summary>
        /// 预约ID
        /// </summary>
        private int _ID;
        public int ID
        {
            get { return _ID; }
            set { _ID = value; RaisePropertyChanged("ID"); }
        }
        /// <summary>
        /// 预约房间ID
        /// </summary>
        private int _OrderRoomID;
        public int OrderRoomID
        {
            get { return _OrderRoomID; }
            set { _OrderRoomID = value; RaisePropertyChanged("OrderRoomID"); }
        }
        /// <summary>
        /// 预约房间名称
        /// </summary>
        private string _OrderRoomName;
        public string OrderRoomName
        {
            get { return _OrderRoomName; }
            set { _OrderRoomName = value; RaisePropertyChanged("OrderRoomName"); }
        }
        /// <summary>
        /// 预约人ID
        /// </summary>
        private string _OrderUserID;
        public string OrderUserID
        {
            get { return _OrderUserID; }
            set { _OrderUserID = value; RaisePropertyChanged("OrderUserID"); }
        }
        /// <summary>
        /// 预约人姓名
        /// </summary>
        private string _OrderUserName;
        public string OrderUserName
        {
            get { return _OrderUserName; }
            set { _OrderUserName = value; RaisePropertyChanged("OrderUserName"); }
        }

        /// <summary>
        /// 审核状态
        /// </summary>
        private ViewOrderTypeEnum _OrderType;
        public ViewOrderTypeEnum OrderType
        {
            get { return _OrderType; }
            set { _OrderType = value; RaisePropertyChanged("OrderType"); }
        }
        /// <summary>
        /// 预约日期
        /// </summary>
        private string _OrderStartTime;
        public string OrderStartTime
        {
            get { return _OrderStartTime; }
            set { _OrderStartTime = value; RaisePropertyChanged("OrderStartTime"); }
        }
        /// <summary>
        /// 星期几
        /// </summary>
        private string _WeekDay;
        public string WeekDay
        {
            get { return _WeekDay; }
            set { _WeekDay = value; RaisePropertyChanged("WeekDay"); }
        }
        /// <summary>
        /// 预约时间段ID
        /// </summary>
        private string _DayTimeID;
        public string DayTimeID
        {
            get { return _DayTimeID; }
            set { _DayTimeID = value; RaisePropertyChanged("DayTimeID"); }
        }
        /// <summary>
        /// 预约时间段
        /// </summary>
        private string _DayTime;
        public string DayTime
        {
            get { return _DayTime; }
            set { _DayTime = value; RaisePropertyChanged("DayTime"); }
        }
        /// <summary>
        /// 预约事由
        /// </summary>
        private string _Reason;
        public string Reason
        {
            get { return _Reason; }
            set { _Reason = value; RaisePropertyChanged("Reason"); }
        }
        public static explicit operator ViewOrder(DashBoard.ServiceModel.Order data)
        {
            List<string> weeks = new List<string>() { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
            if (data == null) return null;
            var entity = new ViewOrder();
            entity.ID = data.ID;
            entity.OrderRoomID = data.OrderRoomID;
            entity.OrderRoomName = data.OrderRoomName;
            entity.OrderUserID = data.OrderUserID;
            entity.OrderUserName = data.OrderUserName;
            entity.OrderType = (ViewOrderTypeEnum)(int)data.OrderType;
            entity.OrderStartTime = data.OrderStartTime.ToString("MM-dd");
            entity.WeekDay = weeks[(int)data.WeedDay];
            entity.DayTimeID = data.DayTimeID;
            entity.DayTime = data.DayTime;
            entity.Reason = data.Reason;
            return entity;
        }

    }

    public enum ViewOrderTypeEnum
    {
        待审核 = 0,
        已预约 = 1,
        未预约 = 2,
        超时取消 = 3,

    }

}
