﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using GalaSoft.MvvmLight;

namespace SchoolSituation
{
    public class ViewStudentState : ObservableObject
    {
        private int _Total;
        /// <summary>
        /// 总计
        /// </summary>
        public int Total
        {
            get { return _Total; }
            set { _Total = value; RaisePropertyChanged("Total"); }
        }

        private int _In;
        /// <summary>
        /// 在校
        /// </summary>
        public int In
        {
            get { return _In; }
            set { _In = value; RaisePropertyChanged("In"); }
        }

        private int _Out;
        /// <summary>
        /// 离校
        /// </summary>
        public int Out
        {
            get { return _Out; }
            set { _Out = value; RaisePropertyChanged("Out"); }
        }


        private ViewGrade _GaoYi;

        public ViewGrade GaoYi
        {
            get { return _GaoYi; }
            set { _GaoYi = value; RaisePropertyChanged("GaoYi"); }
        }

        private ViewGrade _GaoEr;

        public ViewGrade GaoEr
        {
            get { return _GaoEr; }
            set { _GaoEr = value; RaisePropertyChanged("GaoEr"); }
        }

        private ViewGrade _GaoSan;

        public ViewGrade GaoSan
        {
            get { return _GaoSan; }
            set { _GaoSan = value; RaisePropertyChanged("GaoSan"); }
        }

        public static explicit operator ViewStudentState(DashBoard.ServiceModel.School_StudentState data)
        {
            var entity = new ViewStudentState();
            entity.Total = data.StudentCount;
            entity.In = data.InCount;
            entity.Out = data.OutCount;
            entity.GaoYi = (ViewGrade)data.GaoYi;
            entity.GaoEr = (ViewGrade)data.GaoEr;
            entity.GaoSan = (ViewGrade)data.GaoSan;
            return entity;
        }
    }

    public class ViewGrade : ObservableObject
    {
        public static explicit operator ViewGrade(DashBoard.ServiceModel.Grade data)
        {
            var entity = new ViewGrade();
            entity.Total = data.Total;
            entity.In = data.In;
            entity.Out = data.Out;
            entity.ClassList = data.ClassList.Select(ss => (ViewClass)ss).ToList();
            return entity;
        }
        private int _ID;
        public int ID
        { get { return _ID; } set { _ID = value; RaisePropertyChanged("ID"); } }

        private string _Name;
        public string Name
        {
            get { return _Name; }
            set { _Name = value; RaisePropertyChanged("Name"); }
        }
        private int _Total;
        public int Total
        {
            get { return _Total; }
            set { _Total = value; RaisePropertyChanged("Total"); }
        }
        private int _In;
        public int In
        {
            get { return _In; }
            set { _In = value; RaisePropertyChanged("In"); }
        }
        private int _Out;
        public int Out
        {
            get { return _Out; }
            set { _Out = value; RaisePropertyChanged("Out"); }
        }
        private int _Leave;
        public int Leave
        {
            get { return _Leave; }
            set { _Leave = value; RaisePropertyChanged("Leave"); }
        }
        private List<ViewClass> _ClassList;
        public List<ViewClass> ClassList
        {
            get { return _ClassList; }
            set { _ClassList = value; RaisePropertyChanged("ClassList"); }
        }

    }

    public class ViewClass : ObservableObject
    {
        public static explicit operator ViewClass(DashBoard.ServiceModel.Class data)
        {
            var entity = new ViewClass();
            entity.Total = data.Total;
            entity.In = data.In;
            entity.Out = data.Out;
            entity.ClassID = data.ClassID;
            entity.ClassName = data.ClassName;
            return entity;
        }

        private Guid _ClassID;
        public Guid ClassID
        {
            get { return _ClassID; }
            set { _ClassID = value; RaisePropertyChanged("ClassID"); }
        }
        private string _ClassName;
        public string ClassName
        {
            get { return _ClassName; }
            set { _ClassName = value; RaisePropertyChanged("ClassName"); }
        }

        private int _Total;
        public int Total
        {
            get { return _Total; }
            set { _Total = value; RaisePropertyChanged("Total"); }
        }
        private int _In;
        public int In
        {
            get { return _In; }
            set { _In = value; RaisePropertyChanged("In"); }
        }

        private int _Out;
        public int Out
        {
            get { return _Out; }
            set { _Out = value; RaisePropertyChanged("Out"); }
        }
        private int _Leave;
        public int Leave
        {
            get { return _Leave; }
            set { _Leave = value; RaisePropertyChanged("Leave"); }
        }
    }

    public class ViewStudent : ObservableObject
    {
        public static explicit operator ViewStudent(DashBoard.ServiceModel.Student data)
        {
            var entity = new ViewStudent();
            entity.ID = data.ID;
            entity.StudentID = data.StudentID;
            entity.StudentName = data.StudentName;
            entity.InSchool = data.StudentState == DashBoard.ServiceModel.StateEnum.正常;
            entity.StudentState = data.StudentState.ToString();
            entity.RecordTime = data.RecordTime == DateTime.MinValue ? "-" : data.RecordTime.ToString("MM-dd HH:mm:ss");
            entity.Remark = data.Remark;
            return entity;
        }

        public int ID { get; set; }
        public int StudentID { get; set; }
        public bool InSchool { get; set; }
        private string _StudentName;

        public string StudentName
        {
            get { return _StudentName; }
            set { _StudentName = value; RaisePropertyChanged("StudentName"); }
        }

        private string _RecordTime;

        public string RecordTime
        {
            get { return _RecordTime; }
            set { _RecordTime = value; RaisePropertyChanged("RecordTime"); }
        }

        private string _StudentState;

        public string StudentState
        {
            get { return _StudentState; }
            set { _StudentState = value; RaisePropertyChanged("StudentState"); }
        }
        private string _Remark;

        public string Remark
        {
            get { return _Remark; }
            set { _Remark = value; RaisePropertyChanged("Remark"); }
        }


    }
}
