﻿using GalaSoft.MvvmLight.Command;
using GS.Terminal.DashBoard.Interface;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace SchoolSituation
{
    public partial class StudentStateViewModel
    {
        private bool _DetailShowing;
        /// <summary>
        /// 详情窗口是否打开
        /// </summary>
        public bool DetailShowing
        {
            get { return _DetailShowing; }
            set
            {
                _DetailShowing = value;
                RaisePropertyChanged("DetailShowing");
            }
        }

        private bool _ShowInSchool = false;

        public bool ShowInSchool
        {
            get { return _ShowInSchool; }
            set
            {
                _ShowInSchool = value;
                ShowList = new ObservableCollection<ViewStudent>(AllData.Where(ss => ss.InSchool == ShowInSchool));
                RaisePropertyChanged("ShowInSchool");
            }
        }

        private string _TipMsg;

        public string TipMsg
        {
            get { return _TipMsg; }
            set { _TipMsg = value; RaisePropertyChanged("TipMsg"); }
        }
        private List<ViewStudent> AllData = new List<ViewStudent>();
        private ObservableCollection<ViewStudent> _ShowList;

        public ObservableCollection<ViewStudent> ShowList
        {
            get { return _ShowList ?? (_ShowList = new ObservableCollection<ViewStudent>()); }
            set
            {
                _ShowList = value;
                if (value.Count == 0)
                    TipMsg = "并没有找到任何数据...";
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    RaisePropertyChanged("ShowList");
                }));
            }
        }

        private bool _CanCloseDetail = true;

        public bool CanCloseDetail
        {
            get { return _CanCloseDetail; }
            set { _CanCloseDetail = value; RaisePropertyChanged("CanCloseDetail"); }
        }




        private void Loaddata(Guid cid)
        {
            var serviceUrl = new Uri($"{Source}?DataType=Class&ClassID={cid}");
            Task.Run(() =>
            {
                TipMsg = "正在拉取数据...";
                CanCloseDetail = false;
                var result = Tools.Get(serviceUrl, out List<DashBoard.ServiceModel.Student> list);
                if (result)
                {
                    if (list.Count > 0)
                    {
                        AllData = list.Select(ss => (ViewStudent)ss).ToList();
                        ShowList = new ObservableCollection<ViewStudent>(AllData.Where(ss => ss.InSchool == ShowInSchool));
                        if (ShowList.Count == 0)
                        {
                            TipMsg = "并没有找到任何数据...";
                        }
                    }
                    else
                    {
                        TipMsg = "并没有找到任何数据...";
                    }
                }
                else
                {
                    TipMsg = "网络好像出现了异常...";
                }
            }).ContinueWith(task =>
            {
                //TipMsg = "完成";
                CanCloseDetail = true;
            });
        }

        private ViewClass _SelectedClass;

        public ViewClass SelectedClass
        {
            get { return _SelectedClass; }
            set { _SelectedClass = value; RaisePropertyChanged("SelectedClass"); }
        }

        public RelayCommand<ViewClass> ShowDetailCommand => new RelayCommand<ViewClass>(item =>
        {
            if (item != null)
            {
                SelectedClass = item;
                DetailShowing = true;
                Loaddata(item.ClassID);
            }
        });

        public RelayCommand CloseDetailCommand => new RelayCommand(() =>
        {

            DetailShowing = false;
            SelectedClass = null;
            AllData.Clear();
            ShowList.Clear();
            //ShowInSchool = false;
        });

        
    }
}
