﻿using GS.Terminal.DashBoard.Interface;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight.Command;
using Develop.Extension.Wpf;
using System.Windows;
using GalaSoft.MvvmLight;

namespace SchoolSituation
{
    public class TeacherLeaveViewModel : PageViewModel
    {
        private Pager _DayPager;

        public Pager DayPager
        {
            get { return _DayPager ?? (_DayPager = new Pager(7)); }
            set { _DayPager = value; RaisePropertyChanged("DayPager"); }
        }


        public TeacherLeaveViewModel()
        {
            DayPager.OnPageChanged += DayPager_OnPageChanged;
            DayPager.TotalData = 28;
        }

        private void DayPager_OnPageChanged(Pager obj)
        {
            Days.Clear();
            DateTime startDate = DateTime.Now.AddDays((obj.PageIndex - 1) * 7);
            for (int i = 0; i < 7; i++)
            {

                Days.Add(new CustomDay(startDate.AddDays(i)));
            }
            SelectDay.Execute(Days[0]);
        }
        public CustomDay CurrentDay { get; set; }
        public RelayCommand<CustomDay> SelectDay => new RelayCommand<CustomDay>(item =>
        {
            if (!item.IsSelected)
            {
                if (CurrentDay != null) CurrentDay.IsSelected = false;
                item.IsSelected = true;
                CurrentDay = item;
                if (!isfirst)
                    Task.Run(() => { LoadData(true); });
            }

        });
        private ObservableCollection<CustomDay> _Days;

        public ObservableCollection<CustomDay> Days
        {
            get { return _Days ?? (_Days = new ObservableCollection<CustomDay>()); }
            set
            {
                _Days = value;
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    RaisePropertyChanged("Days");
                }));

            }
        }
        public class CustomDay : ObservableObject
        {
            private List<string> weeks = new List<string>() { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
            public CustomDay(DateTime date)
            {
                WeekString = weeks[(int)date.DayOfWeek];
                if (date.Date == DateTime.Now.Date)
                {
                    DateString = "今天";
                }
                else
                {
                    DateString = date.ToString("MM-dd");
                }
                Date = date;
            }

            private string _WeekString;

            public string WeekString
            {
                get { return _WeekString; }
                set { _WeekString = value; RaisePropertyChanged("WeekString"); }
            }

            private string _DateString;

            public string DateString
            {
                get { return _DateString; }
                set { _DateString = value; RaisePropertyChanged("DateString"); }
            }

            private bool _IsSelected;

            public bool IsSelected
            {
                get { return _IsSelected; }
                set { _IsSelected = value; RaisePropertyChanged("IsSelected"); }
            }


            public DateTime Date { get; set; }
        }

        private string _TipMsg;

        public string TipMsg
        {
            get { return _TipMsg; }
            set { _TipMsg = value; RaisePropertyChanged("TipMsg"); }
        }

        private string currentDataMD5;//当前数据的MD5值，用来判断是否更新页面
        private void LoadData(bool claredata = false)
        {
            if (CurrentDay == null) return;
            if (claredata)
            {
                currentDataMD5 = String.Empty;
                TeacherList = new ObservableCollection<ViewTeacherLeave>();
            }
            TipMsg = "正在拉取数据...";
            var ServiceUri = new Uri($"{Source}?NowTime={CurrentDay.Date.ToString("MM-dd")}");
            var result = Tools.Get(ServiceUri, out string content);//TODO:需要计算返回内容的MD5值后判断是否更新数据
            var newMD5 = Tools.GetMD5WithString(content);
            if (result)
            {
                var shouleUpdate = !newMD5.Equals(currentDataMD5, StringComparison.CurrentCultureIgnoreCase);
                currentDataMD5 = newMD5;
                var leaves = JsonConvert.DeserializeObject<List<DashBoard.ServiceModel.Leave>>(content);
                if (leaves.Count > 0)
                {
                    if (shouleUpdate)
                        TeacherList = new ObservableCollection<ViewTeacherLeave>(leaves.Select(ss => (ViewTeacherLeave)ss));
                    TipMsg = "完成";
                }
                else
                {
                    if (shouleUpdate)
                        TeacherList = new ObservableCollection<ViewTeacherLeave>();
                    TipMsg = "没有找到教师假条信息...";
                }
            }
            else
            {
                TipMsg = "网络异常，数据获取失败...";
            }
        }

        private ObservableCollection<ViewTeacherLeave> _TeacherList;
        public ObservableCollection<ViewTeacherLeave> TeacherList
        {
            get { return _TeacherList; }
            set { _TeacherList = value; RaisePropertyChanged("TeacherList"); }
        }
        public override void Enter()
        {
            throw new NotImplementedException();
        }

        public override void Leave()
        {
            throw new NotImplementedException();
        }


        private bool isfirst = true;
        public override bool Update()
        {
            LoadData();
            isfirst = false;
            return false;
        }
    }
}
